// 
// Copyright (C) 2007, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//
#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// R-squared indicator from the article 'Confirming Price Trend' authored by Barbara Star, PhD in the December 2007 issue of S+C
    /// </summary>
    [Description("R-squared indicator")]
    public class Rsquared : Indicator
    {
        #region Variables
		private int period = 8; // Default setting for Period
		private	DataSeries		y;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.Red, PlotStyle.Line, "R-Squared"));
			Add(new Line(Color.DarkViolet, 0.2, "Lower"));
			Add(new Line(Color.YellowGreen, 0.75, "Upper"));
			
			y					= new DataSeries(this);
			
            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			double	sumX	= (double) Period * (Period - 1) * 0.5;
			double	divisor = sumX * sumX - (double) Period * Period * (Period - 1) * (2 * Period - 1) / 6;
			double	sumXY	= 0;
			double	sumX2	= 0;
			double	sumY2	= 0;

			for (int count = 0; count < Period && CurrentBar - count >= 0; count++)
			{
				sumXY += count * Input[count];
				sumX2 += (count * count);
				sumY2 += (Input[count] * Input[count]);
			}
			
			y.Set(Input[0]);
			double	slope		= ((double) Period * sumXY - sumX * SUM(y, Period)[0]) / divisor;
			double	intercept	= (SUM(y, Period)[0] - slope * sumX) / Period;
			
			Rsquare.Set(Math.Pow((Period * sumXY - sumX * SUM(y, Period)[0]) /	Math.Sqrt( (Period * sumX2- (sumX*sumX)) * (Period*sumY2 - (SUM(y, Period)[0]*SUM(y, Period)[0])) ) , 2));
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Rsquare
        {
            get { return Values[0]; }
        }

        [Description("Numbers of bars used for calculations")]
        [Category("Parameters")]
        public int Period
        {
            get { return period; }
            set { period = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private Rsquared[] cacheRsquared = null;

        private static Rsquared checkRsquared = new Rsquared();

        /// <summary>
        /// R-squared indicator
        /// </summary>
        /// <returns></returns>
        public Rsquared Rsquared(int period)
        {
            return Rsquared(Input, period);
        }

        /// <summary>
        /// R-squared indicator
        /// </summary>
        /// <returns></returns>
        public Rsquared Rsquared(Data.IDataSeries input, int period)
        {
            checkRsquared.Period = period;
            period = checkRsquared.Period;

            if (cacheRsquared != null)
                for (int idx = 0; idx < cacheRsquared.Length; idx++)
                    if (cacheRsquared[idx].Period == period && cacheRsquared[idx].EqualsInput(input))
                        return cacheRsquared[idx];

            Rsquared indicator = new Rsquared();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Period = period;

            Rsquared[] tmp = new Rsquared[cacheRsquared == null ? 1 : cacheRsquared.Length + 1];
            if (cacheRsquared != null)
                cacheRsquared.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheRsquared = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// R-squared indicator
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Rsquared Rsquared(int period)
        {
            return _indicator.Rsquared(Input, period);
        }

        /// <summary>
        /// R-squared indicator
        /// </summary>
        /// <returns></returns>
        public Indicator.Rsquared Rsquared(Data.IDataSeries input, int period)
        {
            return _indicator.Rsquared(input, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// R-squared indicator
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Rsquared Rsquared(int period)
        {
            return _indicator.Rsquared(Input, period);
        }

        /// <summary>
        /// R-squared indicator
        /// </summary>
        /// <returns></returns>
        public Indicator.Rsquared Rsquared(Data.IDataSeries input, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.Rsquared(input, period);
        }

    }
}
#endregion
